using System.Text.Json;
using System.Text.Json.Serialization;

namespace File_Watcher.Models.Binder;

public class DriveConfiguration
{

    public string? Letter { get; set; }
    public string? Password { get; set; }
    public string? Share { get; set; }
    public bool? Use { get; set; }
    public string? User { get; set; }

    public override string ToString()
    {
        string result = JsonSerializer.Serialize(this, BinderDriveConfigurationSourceGenerationContext.Default.DriveConfiguration);
        return result;
    }

    private static void PreVerify(IConfigurationRoot configurationRoot, DriveConfiguration? driveConfiguration)
    {
        if (driveConfiguration?.Letter is null)
        {
            List<string> paths = [];
            foreach (IConfigurationProvider configurationProvider in configurationRoot.Providers)
            {
                if (configurationProvider is not Microsoft.Extensions.Configuration.Json.JsonConfigurationProvider jsonConfigurationProvider)
                    continue;
                if (jsonConfigurationProvider.Source.FileProvider is not Microsoft.Extensions.FileProviders.PhysicalFileProvider physicalFileProvider)
                    continue;
                paths.Add(physicalFileProvider.Root);
            }
            throw new NotSupportedException($"Not found!{Environment.NewLine}{string.Join(Environment.NewLine, paths.Distinct())}");
        }
    }

    private static void Verify(DriveConfiguration _)
    {
    }

    private static Models.DriveConfiguration Get(DriveConfiguration? driveConfiguration)
    {
        Models.DriveConfiguration result;
        if (driveConfiguration?.Letter is null)
            throw new NullReferenceException(nameof(Letter));
        if (driveConfiguration?.Password is null)
            throw new NullReferenceException(nameof(Password));
        if (driveConfiguration?.Share is null)
            throw new NullReferenceException(nameof(Share));
        if (driveConfiguration?.Use is null)
            throw new NullReferenceException(nameof(Use));
        if (driveConfiguration?.User is null)
            throw new NullReferenceException(nameof(User));
        Verify(driveConfiguration);
        result = new(driveConfiguration.Letter, driveConfiguration.Password, driveConfiguration.Share, driveConfiguration.Use.Value, driveConfiguration.User);
        return result;
    }

    public static Models.DriveConfiguration Get(IConfigurationRoot configurationRoot)
    {
        Models.DriveConfiguration result;
        IConfigurationSection configurationSection = configurationRoot.GetSection(nameof(Models.DriveConfiguration));
#pragma warning disable IL3050, IL2026
        DriveConfiguration? driveConfiguration = configurationSection.Get<DriveConfiguration>();
#pragma warning restore IL3050, IL2026
        PreVerify(configurationRoot, driveConfiguration);
        result = Get(driveConfiguration);
        return result;
    }

}

[JsonSourceGenerationOptions(WriteIndented = true)]
[JsonSerializable(typeof(DriveConfiguration))]
internal partial class BinderDriveConfigurationSourceGenerationContext : JsonSerializerContext
{
}