using System.Text.Json;
using System.Text.Json.Serialization;

namespace File_Watcher.Models.Binder;

public class EDADatabaseConfiguration
{

    public string? CSharpDateTimeFormat { get; set; }
    public string? FileShare { get; set; }
    public string? Name { get; set; }
    public string? OracleDateTimeFormat { get; set; }
    public string? Password { get; set; }
    public string? TNS { get; set; }
    public string? UserName { get; set; }

    public override string ToString()
    {
        string result = JsonSerializer.Serialize(this, BinderEDADatabaseConfigurationSourceGenerationContext.Default.EDADatabaseConfiguration);
        return result;
    }

    private static void PreVerify(IConfigurationRoot configurationRoot, EDADatabaseConfiguration? configuration)
    {
        if (configuration?.CSharpDateTimeFormat is null)
        {
            List<string> paths = [];
            foreach (IConfigurationProvider configurationProvider in configurationRoot.Providers)
            {
                if (configurationProvider is not Microsoft.Extensions.Configuration.Json.JsonConfigurationProvider jsonConfigurationProvider)
                    continue;
                if (jsonConfigurationProvider.Source.FileProvider is not Microsoft.Extensions.FileProviders.PhysicalFileProvider physicalFileProvider)
                    continue;
                paths.Add(physicalFileProvider.Root);
            }
            throw new NotSupportedException($"Not found!{Environment.NewLine}{string.Join(Environment.NewLine, paths.Distinct())}");
        }
    }

    private static void Verify(EDADatabaseConfiguration _)
    {
    }

    private static Models.EDADatabaseConfiguration Get(EDADatabaseConfiguration? configuration)
    {
        Models.EDADatabaseConfiguration result;
        if (configuration is null) throw new NullReferenceException(nameof(configuration));
        if (configuration.CSharpDateTimeFormat is null) throw new NullReferenceException(nameof(configuration.CSharpDateTimeFormat));
        if (configuration.FileShare is null) throw new NullReferenceException(nameof(configuration.FileShare));
        if (configuration.Name is null) throw new NullReferenceException(nameof(configuration.Name));
        if (configuration.OracleDateTimeFormat is null) throw new NullReferenceException(nameof(configuration.OracleDateTimeFormat));
        if (configuration.Password is null) throw new NullReferenceException(nameof(configuration.Password));
        if (configuration.TNS is null) throw new NullReferenceException(nameof(configuration.TNS));
        if (configuration.UserName is null) throw new NullReferenceException(nameof(configuration.UserName));
        Verify(configuration);
        result = new(configuration.CSharpDateTimeFormat,
                     configuration.FileShare,
                     configuration.Name,
                     configuration.OracleDateTimeFormat,
                     configuration.Password,
                     configuration.TNS,
                     configuration.UserName);
        return result;
    }

    public static Models.EDADatabaseConfiguration Get(IConfigurationRoot configurationRoot)
    {
        Models.EDADatabaseConfiguration result;
        IConfigurationSection configurationSection = configurationRoot.GetSection(nameof(Models.EDADatabaseConfiguration));
#pragma warning disable IL3050, IL2026
        EDADatabaseConfiguration? configuration = configurationSection.Get<EDADatabaseConfiguration>();
#pragma warning restore IL3050, IL2026
        PreVerify(configurationRoot, configuration);
        result = Get(configuration);
        return result;
    }

}

[JsonSourceGenerationOptions(WriteIndented = true)]
[JsonSerializable(typeof(EDADatabaseConfiguration))]
internal partial class BinderEDADatabaseConfigurationSourceGenerationContext : JsonSerializerContext
{
}